`timescale 1ps/1ps     

//====================================================================================
// Filename: gxb_state_mc.v
//
// Description: 
// This is a simple state machine that puts the PHY IP through the following states 
// one after the other and toggle the  control inputs accordingly:
// 
//   * RESET_PHASE: PHY IP in reset
//                   
//   * INIT_PHASE:   link initialization phase  
//
//   * DATA_PHASE:  Send and receive data
//     
//     
//     After sending/receiving data for some time, assert ready_for_mgmt while 
//     continuing to be in data_phase. 
//   
//====================================================================================

module gxb_state_mc (                                                                                               
  inclk,   
  reset,
  tx_ready,
  rx_ready,  
  rx_syncstatus,
  init_phase,
  data_phase,
  ready_for_mgmt,
  rx_sync_flag
 
);

input  inclk;   
input  reset;
input [1:0]  rx_syncstatus;
input        tx_ready;
input        rx_ready;
output reg init_phase = 1'b0; 
output reg data_phase = 1'b0;
output reg ready_for_mgmt = 1'b0;
output reg rx_sync_flag = 1'b0;

//---------------------------------------------
// Registers and Wires
//---------------------------------------------

// Counters for every GXB phase
reg [11:0] init_phase_cntr = 12'h000;
reg [11:0] data_phase_cntr = 12'h000; 
reg [1:0]  gxb_state = 2'b00;

// Registers for status outputs from PHY IP
reg        tx_ready_reg;       
reg        rx_ready_reg;
reg [1:0]  rx_syncstatus_reg;  

       
// GXB states
parameter RESET_PHASE = 2'b00;
parameter INIT_PHASE  = 2'b01;
parameter DATA_PHASE  = 2'b10;
                        

//----------------------------------------------------------------------
// State Machine that puts the GXB through the following states:

//   - RESET_PHASE
//   - INIT_PHASE -> Link initialization/synchronization
//     
//   - DATA_PHASE -> Send PRBS data
//       - ready_for_mgmt -> to custom_avalon_master to do memory mapped 
//                           register transactions
//   - Continueto stay in DATA_PHASE until reset is asserted
// 
//----------------------------------------------------------------------

always @ (posedge inclk or posedge reset) 
begin 
  if (reset)
  begin 
    init_phase_cntr <= 12'h000;
    data_phase_cntr <= 12'h000;
    gxb_state      <= RESET_PHASE;
    init_phase      <= 1'b0;
    data_phase      <= 1'b0;
    ready_for_mgmt  <= 1'b0;
    rx_sync_flag    <= 1'b0;
     
   
    tx_ready_reg             <= 1'b0; 
    rx_ready_reg             <= 1'b0; 
    rx_syncstatus_reg        <= 2'b00;
   
  end // reset
  else 
  begin // not reset

    // Register Custom PHY IP outputs
    tx_ready_reg        <= tx_ready; 
    rx_ready_reg        <= rx_ready; 
    rx_syncstatus_reg   <= rx_syncstatus;
 

   case (gxb_state)
      //----------------------------------------
      // RESET_PHASE     
      //----------------------------------------
      RESET_PHASE: 
      begin  
        init_phase_cntr <= 12'h000;
        data_phase_cntr <= 12'h000;
        init_phase      <= 1'b0;
        data_phase      <= 1'b0;
        ready_for_mgmt  <= 1'b0;
        rx_sync_flag    <= 1'b0;
  
      
        // Switch to INIT_PHASE once tx_ready and rx_ready from the 
        // PHY IP's reset controller is asserted 
        if (tx_ready_reg == 1'b1 && rx_ready_reg == 1'b1)
          gxb_state <= INIT_PHASE;
        else
          gxb_state <= RESET_PHASE;
      end
      
      //----------------------------------------
      // INIT_PHASE     
      //----------------------------------------
      INIT_PHASE:
      begin
        data_phase_cntr <= 12'h000;        
        init_phase      <= 1'b1;
        data_phase      <= 1'b0;
        ready_for_mgmt  <= 1'b0;	   
        init_phase_cntr <= init_phase_cntr + 1'b1;
       
        if (reset)
          gxb_state <= RESET_PHASE;
        else
        begin 
          if ((init_phase_cntr >= 12'h000) && (init_phase_cntr <= 12'h050))
          begin
            gxb_state      <= INIT_PHASE;
            
          end //init_phase_cntr >=0 && <= 050
          
          else if ((init_phase_cntr >= 12'h051) && (init_phase_cntr <= 12'h164)) 
	  begin
            gxb_state      <= INIT_PHASE;   	  
         
            // Check for assertion of rx_syncstatus
            //if (rx_syncstatus_reg == 4'b1111)
				  if (rx_syncstatus_reg == 2'b11)// modified for single channel, MC
	     begin	    
                 rx_sync_flag <= 1'b1;
             end
            else 
	     begin
              rx_sync_flag <= 1'b0;   
	     end //init_phase_cntr >= 051 && <= 164
	   end
          
          else if (init_phase_cntr > 12'h164)
           begin		  
            init_phase_cntr <= 12'h164; // Saturate the cntr          
          // Move to data phase after rx is synchronized
          if (rx_sync_flag == 1'b1)
            gxb_state  <= DATA_PHASE;
          else
            gxb_state  <= INIT_PHASE;
          end
        end // not reset
      end //INIT_PHASE
		      		   
      //----------------------------------------
      // DATA_PHASE     
      //----------------------------------------
      DATA_PHASE: 
      begin  	  
       
       init_phase_cntr <= 12'h000;       
       init_phase <= 1'b0;
       data_phase <= 1'b1;

       if (reset) 
         gxb_state <= RESET_PHASE;
       else
       begin 
         gxb_state <= DATA_PHASE;
       end
       
       // Give control to custom_avalon_master after sending
       // data for sometime. Continue to be in DATA_PHASE and 
       // send data during management stage.    
       if (data_phase_cntr >= 12'h264)
       begin
         data_phase_cntr <= 12'h264; //Saturate to avoid wrap around
         ready_for_mgmt <= 1'b1;
       end
       else
       begin
         data_phase_cntr <= data_phase_cntr + 1'b1;
         ready_for_mgmt <= 1'b0;
       end
  
     end //DATA_PHASE          

   endcase
 end // not reset
end // always   
  
endmodule
