/////////////////////////////////////////////////////////////////////////////////////////////////
// Top level file for Analog reconfig IP
// The file instantiates:
// top_custom - custom_phy and top_reconfig - reconfig analog IP
/////////////////////////////////////////////////////////////////////////////////////////////////


module top
#(  parameter ROM_DEPTH         = 512,
    parameter PIO_OUT_SIZE      = 16,
    parameter PIO_IN_SIZE       = 32,
    parameter CLK_PERIOD_IN_PS  = 10000    
    ) 
	 
	 (	 
  // Top level inputs
    input  wire         pll_ref_clk,     // Input reference clock          
	 input  wire 	      clk150,          // Input management clock, reconfig clock of 150 MHz       
    input  wire 	      reset,           // System reset
	 input  wire         rx_serial_data1, // RX serial input for XCVR ch1       
	 input  wire         rx_serial_data2, // RX serial input for XCVR ch2
	
	 
  // Top level outputs	 
    output wire         tx_clkout1,      // TX parallel clock for XCVR ch1           
	 output wire         tx_clkout2,      // TX parallel clock for XCVR ch2          
    output wire         rx_clkout1,      // RX parallel clock for XCVR ch1          
	 output wire         rx_clkout2,      // RX parallel clock for XCVR ch2       
    output wire         tx_ready,        // TX reset sequence status output 
    output wire         tx_serial_data1, // TX serial output for XCVR ch1      
	 output wire         tx_serial_data2, // TX serial output for XCVR ch2      
    output wire         rx_ready,        // RX reset sequence status output     
    output wire 	      reconfig_busy,   // Reconfig controller busy status
	 output wire     		init_phase,      // gxb_state_mc output - start of initialization phase
	 output wire			data_phase,	     // gxb_state_mc output - start of data phase
    output wire         ready_for_mgmt_and_reconfig,	 // gxb_state_mc output - start of PHY MGMT & RECONFIG cycle
	 output wire [1:0]   rx_syncstatus_read1,
	 output wire [1:0]   rx_syncstatus_read2,
	 output wire [15:0]  pio_out,
	 output wire         prbs_checker_lock,	//indicates prbs checker has locked
	 output wire         prbs_checker_errorFlag // data error
	 );	 
                 
//----------------------------------------------
// Parameters
//----------------------------------------------

 localparam  CLK_PER         = CLK_PERIOD_IN_PS;
 localparam  HALF_CLK_PER    = CLK_PER / 2;
 localparam  CLKS_PER_SECOND = 1000000 / (CLK_PERIOD_IN_PS / 1000);
	
	
//-----------------------------------------------
// Wires                                               
//-----------------------------------------------
	
// To/from transceiver reconfiguration controller
wire [279:0] reconfig_to_xcvr;
wire [183:0]  reconfig_from_xcvr;
	
// To/from Custom PHY IP  instance 1 
wire [15:0]  rx_parallel_data1; 
wire [1:0]   rx_patterndetect1;     
wire [1:0]   rx_syncstatus1;        
wire 			 rx_ready1;
wire 			 tx_ready1;
wire         pll_locked1;
wire [1:0]   rx_runningdisp1;			 
wire 			 rx_byteordflag1;
wire [1:0]   rx_datak1;

// To/from Custom PHY IP  instance 2	
wire [15:0]  rx_parallel_data2;
wire [1:0]   rx_patterndetect2;     
wire [1:0]   rx_syncstatus2;        
wire 			 rx_ready2;
wire 			 tx_ready2;
wire         pll_locked2;
wire [1:0]   rx_runningdisp2;			 
wire 			 rx_byteordflag2;
wire [1:0]   rx_datak2;			 

assign 	 	 rx_ready = rx_ready1 & rx_ready2; 
assign 	 	 tx_ready = tx_ready1 & tx_ready2;

wire 	[1:0]	 rx_patterndetect;
wire 	[1:0]	 rx_syncstatus;
assign		 rx_patterndetect = rx_patterndetect1 & rx_patterndetect2;
assign		 rx_syncstatus		= rx_syncstatus1 & rx_syncstatus2;


         
	
// To/from prbs_generator	
wire [15:0]   prbs_data;	

// PHY management signals (Avalon interface)   
  
 wire [8:0] 	phy_mgmt_address1;     //           phy_mgmt.address
 wire          phy_mgmt_read1;        //                   .read
 wire [31:0] 	phy_mgmt_readdata1;    //                   .readdata
 wire          phy_mgmt_write1;       //                   .write
 wire [31:0] 	phy_mgmt_writedata1;   //                   .writedata
 wire          phy_mgmt_waitrequest1; //                   .waitrequest	
 
 wire [8:0] 	phy_mgmt_address2;     //           phy_mgmt.address
 wire          phy_mgmt_read2;        //                   .read
 wire [31:0] 	phy_mgmt_readdata2;    //                   .readdata
 wire          phy_mgmt_write2;       //                   .write
 wire [31:0] 	phy_mgmt_writedata2;   //                   .writedata
 wire          phy_mgmt_waitrequest2; //                   .waitrequest	
	 
// reconfig management signals	 
 wire [6:0]     reconfig_mgmt_address;
 wire           reconfig_mgmt_read;
 wire           reconfig_mgmt_write;
 wire [31:0]    reconfig_mgmt_writedata;
 wire           reconfig_mgmt_waitrequest;
 wire [31:0]    reconfig_mgmt_readdata;
 wire				 pio_in;
	
 wire [17:0]   init_data_lane1;
 wire [17:0]   init_data_lane2;
 wire [17:0]   init_data_lane3;
 wire [17:0]   init_data_lane4;
 
 wire [11:0]   input_data_size;
 wire          xmit_TS1;

    
wire       rx_sync_flag;  // successful Rx syncronization during init_phase.
                                             
// output from custom_avalon_master
// rx_syncstatus bits read from PCS status register. Note that this is not a live signal 
// because the rx_syncstatus would have gone high long before this PCS register is read from.

 //wire [1:0] rx_syncstatus_read1; 
 //wire [1:0] rx_syncstatus_read2; 

//-----------------------------------------------
// Registers
//-----------------------------------------------

	reg [17:0] txdata_lane1_reg = 18'h00000;
	reg [17:0] txdata_lane2_reg = 18'h00000;
	reg [17:0] txdata_lane3_reg = 18'h00000;
	reg [17:0] txdata_lane4_reg = 18'h00000;
	

//----------------------------------------------------- 
// Assign COM data from syncgen or 
// prbs data from prbs_generator to the parallel txdata
// of the PHY IP 
//------------------------------------------------------

always @ (posedge tx_clkout1 or posedge reset)
begin
  if (reset) 
  begin
    txdata_lane1_reg[17]   <= init_data_lane1[17];
    txdata_lane1_reg[16:9] <= init_data_lane1[16:9];
    txdata_lane1_reg[8]    <= init_data_lane1[8];
    txdata_lane1_reg[7:0]  <= init_data_lane1[7:0];
    
    txdata_lane2_reg[17]   <= init_data_lane2[17];
    txdata_lane2_reg[16:9] <= init_data_lane2[16:9];
    txdata_lane2_reg[8]    <= init_data_lane2[8];
    txdata_lane2_reg[7:0]  <= init_data_lane2[7:0];
    
    txdata_lane3_reg[17]   <= init_data_lane3[17];
    txdata_lane3_reg[16:9] <= init_data_lane3[16:9];
    txdata_lane3_reg[8]    <= init_data_lane3[8];
    txdata_lane3_reg[7:0]  <= init_data_lane3[7:0];
    
    txdata_lane4_reg[17]   <= init_data_lane4[17];
    txdata_lane4_reg[16:9] <= init_data_lane4[16:9];
    txdata_lane4_reg[8]    <= init_data_lane4[8];
    txdata_lane4_reg[7:0]  <= init_data_lane4[7:0];
  end
  else
  begin
    if (data_phase) 
    begin
      txdata_lane1_reg [17]   <= 1'b0;
      txdata_lane1_reg [16:9] <= prbs_data[15:8];
      txdata_lane1_reg [8]    <= 1'b0;
      txdata_lane1_reg [7:0]  <= prbs_data[7:0];

      txdata_lane2_reg [17]   <= 1'b0;
      txdata_lane2_reg [16:9] <= prbs_data[15:8];
      txdata_lane2_reg [8]    <= 1'b0;
      txdata_lane2_reg [7:0]  <= prbs_data[7:0];

      txdata_lane3_reg [17]   <= 1'b0;
      txdata_lane3_reg [16:9] <= prbs_data[15:8];
      txdata_lane3_reg [8]    <= 1'b0;
      txdata_lane3_reg [7:0]  <= prbs_data[7:0];

      txdata_lane4_reg [17]   <= 1'b0;
      txdata_lane4_reg [16:9] <= prbs_data[15:8];
      txdata_lane4_reg [8]    <= 1'b0;
      txdata_lane4_reg [7:0]  <= prbs_data[7:0];
    end
    else
    begin
      txdata_lane1_reg[17]   <= init_data_lane1[17];
      txdata_lane1_reg[16:9] <= init_data_lane1[16:9];
      txdata_lane1_reg[8]    <= init_data_lane1[8];
      txdata_lane1_reg[7:0]  <= init_data_lane1[7:0];
      
      txdata_lane2_reg[17]   <= init_data_lane2[17];
      txdata_lane2_reg[16:9] <= init_data_lane2[16:9];
      txdata_lane2_reg[8]    <= init_data_lane2[8];
      txdata_lane2_reg[7:0]  <= init_data_lane2[7:0];
      
      txdata_lane3_reg[17]   <= init_data_lane3[17];
      txdata_lane3_reg[16:9] <= init_data_lane3[16:9];
      txdata_lane3_reg[8]    <= init_data_lane3[8];
      txdata_lane3_reg[7:0]  <= init_data_lane3[7:0];
      
      txdata_lane4_reg[17]   <= init_data_lane4[17];
      txdata_lane4_reg[16:9] <= init_data_lane4[16:9];
      txdata_lane4_reg[8]    <= init_data_lane4[8];
      txdata_lane4_reg[7:0]  <= init_data_lane4[7:0];
    end
  end
end


//------------------------------------------------------------------
// Instantiate the Custom PHY IP1`

// Number of Lanes = 1
// Protocol Version = Custom 
// Deserialization Factor = 16
// PLL ref clock frequency = 250 MHz 
//------------------------------------------------------------------
   
   
	top_custom top_custom_inst1 (
		  .phy_mgmt_clk(clk150),         
		  .phy_mgmt_clk_reset(reset),                     
		  .phy_mgmt_address( phy_mgmt_address1),     
		  .phy_mgmt_read( phy_mgmt_read1),           
		  .phy_mgmt_readdata( phy_mgmt_readdata1),       
		  .phy_mgmt_write( phy_mgmt_write1),       
		  .phy_mgmt_writedata( phy_mgmt_writedata1),                           
		  .phy_mgmt_waitrequest(phy_mgmt_waitrequest1),   
		  .reconfig_to_xcvr(reconfig_to_xcvr[139:0]),
		  .reconfig_from_xcvr(reconfig_from_xcvr[91:0]),       
		  .tx_ready(tx_ready1),
		  .rx_ready(rx_ready1),		   
		  .tx_serial_data(tx_serial_data1),       
		  .pll_locked(pll_locked1),           
		  .rx_serial_data(rx_serial_data1),       
		  .rx_runningdisp(rx_runningdisp1), 
		  .rx_byteordflag (rx_byteordflag1),
		  .rx_patterndetect(rx_patterndetect1),
		  .rx_syncstatus(rx_syncstatus1),
        .tx_parallel_data    ({txdata_lane1_reg[16:9], txdata_lane1_reg[7:0]}),              
        .tx_datak            ({txdata_lane1_reg[17], txdata_lane1_reg[8]}), 						
		  .rx_parallel_data(rx_parallel_data1),                              
		  .rx_datak(rx_datak1),             
		  .pll_ref_clk(pll_ref_clk),
        .tx_clkout(tx_clkout1),         
        .rx_clkout(rx_clkout1) 		 
	);                                                  
 
 
 //------------------------------------------------------------------
// Instantiate the Custom PHY IP 2 

// Number of Lanes = 1
// Protocol Version = Custom 
// Deserialization Factor = 16
// PLL ref clock frequency = 250 MHz 
//------------------------------------------------------------------
   
   
	top_custom top_custom_inst2 (
		  .phy_mgmt_clk(clk150),         
		  .phy_mgmt_clk_reset(reset),                     
		  .phy_mgmt_address( phy_mgmt_address2),     
		  .phy_mgmt_read( phy_mgmt_read2),           
		  .phy_mgmt_readdata( phy_mgmt_readdata2),       
		  .phy_mgmt_write( phy_mgmt_write2),       
		  .phy_mgmt_writedata( phy_mgmt_writedata2),                           
		  .phy_mgmt_waitrequest(phy_mgmt_waitrequest2),   
		  .reconfig_to_xcvr(reconfig_to_xcvr[279:140]),
		  .reconfig_from_xcvr(reconfig_from_xcvr[183:92]),       
		  .tx_ready(tx_ready2),
		  .rx_ready(rx_ready2),		   
		  .tx_serial_data(tx_serial_data2),       
		  .pll_locked(pll_locked2),           
		  .rx_serial_data(rx_serial_data2),       
		  .rx_runningdisp(rx_runningdisp2), 
		  .rx_byteordflag (rx_byteordflag2),
		  .rx_patterndetect(rx_patterndetect2),
		  .rx_syncstatus(rx_syncstatus2),	
        .tx_parallel_data    ({txdata_lane2_reg[16:9], txdata_lane2_reg[7:0]}),              
        .tx_datak            ({txdata_lane2_reg[17], txdata_lane2_reg[8]}), 						
		  .rx_parallel_data(rx_parallel_data2 ),                              
		  .rx_datak(rx_datak2),             
		  .pll_ref_clk(pll_ref_clk),
        .tx_clkout(tx_clkout2),         
        .rx_clkout(rx_clkout2) 		 
	); 
//--------------------------------------------------------------------------------
// Instantiate the Transceiver Reconfiguration Controller 11.0
//--------------------------------------------------------------------------------     

top_reconfig inst_reconfig 
	(
	 .reconfig_busy(reconfig_busy),
	 .mgmt_clk_clk(clk150),
	 .mgmt_rst_reset(reset),  
	 .reconfig_mgmt_address(reconfig_mgmt_address),
	 .reconfig_mgmt_read(reconfig_mgmt_read),
	 .reconfig_mgmt_write(reconfig_mgmt_write),
	 .reconfig_mgmt_writedata(reconfig_mgmt_writedata),	 
	 .reconfig_mgmt_readdata(reconfig_mgmt_readdata),	 
	 .reconfig_mgmt_waitrequest(reconfig_mgmt_waitrequest),
	 .ch0_1_to_xcvr (reconfig_to_xcvr[139:0]),             
	 .ch0_1_from_xcvr (reconfig_from_xcvr[91:0]),
	 .ch2_3_to_xcvr(reconfig_to_xcvr[279:140]),
	 .ch2_3_from_xcvr (reconfig_from_xcvr[183:92]) 
	 );
	 
	 
//------------------------------------------------------
// Instantiate the GXB state machine
//------------------------------------------------------
// This is a simple state machine that puts the PHY IP through the following states 
// one after the other and toggle the  control inputs accordingly:
// 
//   * RESET_PHASE: PHY IP in reset                   
//   * INIT_PHASE:   link initialization phase  
//   * DATA_PHASE:  Send and receive data//     
//     After sending/receiving data for some time, assert ready_for_mgmt while 
//     continuing to be in data_phase. 
//-----------------------------------------------------------------------------------

gxb_state_mc gxb_state_mc_inst(                                                                                               
  .inclk (tx_clkout1),   
  .reset (reset),
  .tx_ready (tx_ready),
  .rx_ready (rx_ready),  
  .rx_syncstatus (rx_syncstatus), 
  .init_phase (init_phase),
  .data_phase (data_phase),
  .ready_for_mgmt (ready_for_mgmt_and_reconfig),
  .rx_sync_flag (rx_sync_flag) 
);	 

//------------------------------------------------------
// Instantiate the datagen controller 
//------------------------------------------------------
// Based on the inputs from the testbench, the datagen controller transmits 
// control signals to the 16-bit sync generator. In this case, there are only two
// inputs from the testbench: 1) init_phase & 2) data_phase
// During the init_phase, the datagen controller instructs the sync gen to transmit
// COM characters to the TX parallel i/p.
// During the data_phase, the prbs generator kicks in to transmit payload to the
// TX parallel i/p.
// Under all circumstances, the datagenerator blocks use the 'tx_clkout1' to generate
// data to the TX parallel i/p.

datagen_controller datagen_controller_inst 
( 
  .inclk(tx_clkout1),  
  .datagen_reset(reset),
  .init_phase(init_phase),
  .data_phase(data_phase),
//.mgmt_phase(ready_for_mgmt_and_reconfig), //currently not used in this module. Commenting it out
  .xmit_TS1(xmit_TS1),         
  .input_data_size(input_data_size)  
);

//------------------------------------------------------
// Instantiate the sync generator
//------------------------------------------------------
// This module generates the COM characters to the TX parallel i/p.
// This is to achieve word alignment in the RX side. In this case
// internal loopback is enabled.  This module waits for the xmit_TS1 control
// signal from the datagen controller and then transmits COMs.

syncgen_16bit_4lanes syncgen_16bit_4lanes_inst(
  .clk               (tx_clkout1),
  .reset             (reset),           
  .xmit_TS1          (xmit_TS1),        
  .input_data_size   (input_data_size),     
  .data_out_l1       (init_data_lane1),           
  .data_out_l2       (init_data_lane2),        
  .data_out_l3       (init_data_lane3),         
  .data_out_l4       (init_data_lane4)         
);
	 
//------------------------------------------------------
// Instantiate PRBS generator  
//------------------------------------------------------
// The PRBS generator waits for the data_phase control input
// from the testbench, and then transmits PRBS data as payload
// to the transceiver link.

prbs_generator 
 # (
    .PRBS_INITIAL_VALUE (97), //16'h0061
    .DATA_WIDTH (16),
    .PRBS (23)
  ) prbs_gen_inst
  (
    .clk                   (tx_clkout1),
    .nreset                (data_phase), //keep the generator in reset until data_phase is asserted 
    .xaui_word_align       (1'b1),       //active-low signal 
    .insert_error          (1'b0), 
    .pause                 (1'b0), 
    .dout                  (prbs_data)
  );
   
//------------------------------------------------------------------
// Instantiate PRBS checker  
//------------------------------------------------------------------
// We need two checkers because of the uncertainty in the 
// byte order of pipe_rxdata. Depending on the sampling time of 
// the Byte SERDES in the Rx PCS, pipe_rxdata either has bytes 
// ordered correctly or not. Therefore, byte ordering is done earlier 
// in this module to correct for this misordering. 
// We need to check for the above 2 cases.

// Only one of the checkers will lock to the initial data. OR the lock signals.
//assign prbs_checker_lock = prbs_checker_normal_lock | prbs_checker_byte_slip_lock;

// Only one of the checkers will check errors. OR the errors.
//assign prbs_checker_errorFlag = prbs_checker_normal_errorFlag | prbs_checker_byte_slip_errorFlag;

//------------------------------------------------------------------
// Checker for pipe_rxdata with correct byte ordering
//------------------------------------------------------------------

prbs_checker_x4 
#(
  .NUM_CYCLES_FOR_LOCK (0),		
  .PRBS_INITIAL_VALUE (97), //Lock to 97 (16'h0061) (prbs_generator initial value) 
  .DATA_WIDTH (16),
  .PRBS (23)
  ) prbs_checker_x4_inst_normal
    (
    .dataIn_lane1 (rx_parallel_data1), 
    .dataIn_lane2 (rx_parallel_data2), 
    .dataIn_lane3 (), 
    .dataIn_lane4 (), 
    .lock (prbs_checker_lock), 
    .errorFlag (prbs_checker_errorFlag),
    .clk (tx_clkout1), 
    .nreset (data_phase), 
    .pause (1'b0)
   );



mgmt_master1 
#(
    .CLOCKS_PER_SECOND (125000000),  // Used for time calculations
    .PIO_OUT_SIZE      (2),          // Width of PIO output port
    .PIO_IN_SIZE       (1),          // Width of PIO input port
    .ROM_DEPTH         (1024)        // Depth of command ROM
 ) custom_phy_master_inst1 
 (
    .clk             (clk150), 
    .reset           (~ready_for_mgmt_and_reconfig),             
    .av_write        (phy_mgmt_write1),                                                      
    .av_read         (phy_mgmt_read1),                
    .av_address      (phy_mgmt_address1), //phy_mgmt_address=9-bits, av_address=31-bits         
    .av_writedata    (phy_mgmt_writedata1),        
    .av_readdata     (phy_mgmt_readdata1),        
    .av_waitrequest  (phy_mgmt_waitrequest1),
    .pio_in          (ready_for_mgmt_and_reconfig),
    .pio_out         (rx_syncstatus_read1)       
 ); 	
 
 mgmt_master2 
#(
    .CLOCKS_PER_SECOND (125000000),  // Used for time calculations
    .PIO_OUT_SIZE      (2),          // Width of PIO output port
    .PIO_IN_SIZE       (1),          // Width of PIO input port
    .ROM_DEPTH         (1024)        // Depth of command ROM
 ) custom_phy_master_inst2 
 (
    .clk             (clk150), 
    .reset           (~ready_for_mgmt_and_reconfig),             
    .av_write        (phy_mgmt_write2),                                                      
    .av_read         (phy_mgmt_read2),                
    .av_address      (phy_mgmt_address2), //phy_mgmt_address=9-bits, av_address=31-bits         
    .av_writedata    (phy_mgmt_writedata2),        
    .av_readdata     (phy_mgmt_readdata2),        
    .av_waitrequest  (phy_mgmt_waitrequest2),
    .pio_in          (ready_for_mgmt_and_reconfig),
    .pio_out         (rx_syncstatus_read2)       
 ); 	
 
//--------------------------------------------------------------------
// Avalon Master that communicates to the Transceiver Reconfiguration Controller
//--------------------------------------------------------------------

  // wire [PIO_OUT_SIZE-1:0] pio_out;

   mgmt_master3 
     #(
       .CLOCKS_PER_SECOND(CLKS_PER_SECOND),  // Used for time calculations
       .PIO_OUT_SIZE     (PIO_OUT_SIZE   ),  // Width of PIO output port
       .PIO_IN_SIZE      (PIO_IN_SIZE    ),  // Width of PIO input port
       .ROM_DEPTH        (ROM_DEPTH      )  // Depth of command ROM
       //     
       ) custom_reconfig_master_inst (
	      .clk            (clk150),
	      .reset          (~ready_for_mgmt_and_reconfig),
	      
	      .av_write       (reconfig_mgmt_write       ),
	      .av_read        (reconfig_mgmt_read        ),
	      .av_address     (reconfig_mgmt_address     ),
	      .av_writedata   (reconfig_mgmt_writedata   ),
	      .av_readdata    (reconfig_mgmt_readdata    ),
	      .av_waitrequest (reconfig_mgmt_waitrequest ),
	      
	      .pio_out        (pio_out),
	      .pio_in         (pio_in)
	      );
 

 
endmodule // top

     
      
      
