///////////////////////////////////////////////////////////////////////////////////////////
//
// This is the top level testbench for this Custom PHY IP reference design example
// This file instantiates the following design file
// 1) top.sv which is a DUT 
// 
// The testbench generates clocks and reset for the DUT and checks for the status and error 
// signals if any and displays the required info and error messages.
// 
// This design is intended to be simulated in modelsim. 
// To compile and simulate this design, run "vsim -c -do phy_sim_top.tcl" 
//
///////////////////////////////////////////////////////////////////////////////////////////

`timescale 1ps/1ps

module top_tb 

#(      
       parameter PIO_OUT_SIZE      = 16  // Width of PIO output port
  );

   import altera_xcvr_functions::*;
   import alt_xcvr_reconfig_h::*;
   
   // DUT inputs
   reg 	       brd_reset;             
   reg          clk100;
   reg	       clk150;
   reg	       clk125;
   reg	       clk250;
   
   // DUT outputs
   wire         tx_ready;
   wire         rx_ready;
   
   wire         init_phase;
   wire         data_phase;
   wire         ready_for_mgmt;
   wire [1:0]   rx_syncstatus_read1;
   wire [1:0]   rx_syncstatus_read2;
	wire         prbs_checker_lock;
   wire         prbs_checker_errorFlag;
	wire         reconfig_busy;
	wire         tx_clkout1;
	wire         tx_clkout2;
	wire         rx_clkout1;
	wire         rx_clkout2;
	

   // External Serial Loopback 
   wire         tx_serial_data1_lpbk;
   wire         tx_serial_data2_lpbk;
	

   wire [PIO_OUT_SIZE-1:0] pio_out;
   reg 	  test_f = 1'b0; 
   integer  reg_out_file;         
   
   //------------------------------------------------- 
   // Instantiate the top-level reference design 
   //------------------------------------------------- 
   top top_inst
   (
	      .pll_ref_clk                 (clk250),
         .clk150                    (clk150),
         .reset                      (brd_reset),
         .tx_ready                   (tx_ready),
	      .rx_ready                   (rx_ready),
	      .init_phase                 (init_phase),
         .data_phase                 (data_phase), 
         .ready_for_mgmt_and_reconfig(ready_for_mgmt),          
         .tx_serial_data1            (tx_serial_data1_lpbk),
         .rx_serial_data1            (tx_serial_data1_lpbk),
			.tx_serial_data2            (tx_serial_data2_lpbk),
         .rx_serial_data2            (tx_serial_data2_lpbk),
			.tx_clkout1                 (tx_clkout1),           // TX parallel clock for XCVR ch1           
	      .tx_clkout2                 (tx_clkout2),           // TX parallel clock for XCVR ch2          
         .rx_clkout1                 (rx_clkout1),           // RX parallel clock for XCVR ch1          
	      .rx_clkout2                 (rx_clkout2),           // RX parallel clock for XCVR ch2    
         .rx_syncstatus_read1        (rx_syncstatus_read1),
         .rx_syncstatus_read2        (rx_syncstatus_read2),
			.pio_out                    (pio_out),
         .reconfig_busy              (reconfig_busy),   // Reconfig controller busy status
         .prbs_checker_lock          (prbs_checker_lock),
         .prbs_checker_errorFlag     (prbs_checker_errorFlag)
   );	 

   //------------------------------------------------- 
   // Generate reset and set-up output log file 
   //------------------------------------------------- 
   initial
     begin
	reg_out_file = $fopen("sim.out");
	$fdisplay(reg_out_file, "Modelsim Simulation Test");            		
	$fdisplay(reg_out_file, "========================\n");            		
	$fdisplay(reg_out_file, "Custom PHY IP1 & IP2 with PMA Controls Reconfiguration - 1 channel per PHY IP with external serial loopback.\n");
	$fdisplay(reg_out_file, "Modelsim Simulation Result:");
	$fdisplay(reg_out_file, "===========================\n");            		
	
        // Initialize clocks
	clk100 = 1'b0; 
	clk125 = 1'b0;
	clk150 = 1'b0;
	clk250 = 1'b0;

        // Generate reset	
	brd_reset = 1'b0;

        #200;
	brd_reset = 1'b1;
	#20000; 
	brd_reset = 1'b0;

	#20000000; 
	
   test_f = 1'b0;
	// run for 10 us 
	#8000000;
	test_f = 1'b1;
	#90000000;	
		
	$fclose(reg_out_file);	
	//$finish;
	
     end // initial begin

   //------------------------------------------------- 
   // Generate clocks
   //------------------------------------------------- 
   always
     #4000 clk125 <= ~clk125;
   
   always 
     #3333 clk150 <= ~clk150;
   
   always 
     #2000 clk250 <= ~clk250;
  
   always 
     #5000 clk100 <= ~clk100;

      
   //------------------------------------------------- 
   // Tx Ready Status 
   //------------------------------------------------- 
   always @(tx_ready)
     begin
	if (tx_ready)
	  begin
	     $display("\nInfo: TX PCS is ready at %0t ps\n", $realtime);	     
	     $fdisplay(reg_out_file, "Info: TX PCS is ready\n");
	  end
	else
	  $display("Info: TX PCS is not ready\n");	  
     end   

   //------------------------------------------------- 
   // Rx Ready Status 
   //------------------------------------------------- 
   always @(rx_ready)
     begin
	if (rx_ready)
	  begin	     
	     $display("\nInfo: RX PCS is ready at %0t ps\n", $realtime);
	     $fdisplay(reg_out_file, "Info: RX PCS is ready\n");	     
	  end
	else
	  $display("Info: RX PCS is not ready\n");	  
     end

   //------------------------------------------------- 
   // Initialization, p1_to_p0, and data Phase - Gen1
   //------------------------------------------------- 
   always @(init_phase, data_phase)
   begin
     if (init_phase)
     begin
       $display("Info: Initialization Phase starting at %0t ps. ...\n", $realtime);	     
      // $fdisplay(reg_out_file, "Info: Initialization Phase starting at %0t ps....\n\n");
     end
      
     if (data_phase)
     begin
       $display("Info: RX_SYNCSTATUS asserted for both PHY IPs... \n", $realtime);	     
       $fdisplay(reg_out_file, "Info: RX_SYNCSTATUS asserted for both PHY IPs... \n\n");
       #1 $display("Info: Data Phase starting at %0t ps. Sending PRBS data......\n", $realtime);	     
       #1 $fdisplay(reg_out_file, "Info: Starting Data Phase. Sending PRBS data......\n\n");
      end
   end

         
   //------------------------------------------------- 
   // PRBS checker status 
   //------------------------------------------------- 
   always  @ (data_phase, prbs_checker_lock, prbs_checker_errorFlag)
   begin
     if (data_phase)
     begin
       if (prbs_checker_lock == 1'b1)
       begin
           $display("Info: PRBS checker is locked at %0t ps\n", $realtime);	     
           $fdisplay(reg_out_file, "Info: PRBS checker is locked\n");	     
           $display("Info: Continue sending PRBS data......\n\n");	     
           $fdisplay(reg_out_file, "Info: Continue sending PRBS data......\n\n");
           if (prbs_checker_errorFlag == 1'b1)
           begin
             $display("ERROR: PRBS data error in rx_parallel_data1 & rx_parallel_data2 at %0t ps\n", $realtime);	     
             $fdisplay(reg_out_file, "ERROR: PRBS data error in rx_parallel_data1 & rx_parallel_data2\n");	     
           end
       end
       else 
       begin
           #2 $display("Info: PRBS checker is not locked at %0t ps\n", $realtime);	     
           #2 $fdisplay(reg_out_file, "Info: PRBS checker is not locked\n");	     
       end
       if (prbs_checker_errorFlag == 1'b1)
       begin
           $display("ERROR: PRBS data error in rx_parallel_data1 & rx_parallel_data2 at %0t ps\n", $realtime);	     
           $fdisplay(reg_out_file, "ERROR: PRBS data error in rx_parallel_data1 & rx_parallel_data2\n");	     
       end
     end
   end

   

   //------------------------------------------------- 
   //Maganement Register access
   //------------------------------------------------- 
   always @ (ready_for_mgmt, test_f)
   begin
     if (ready_for_mgmt)
     begin
           $display("Info: Accessing PHY management registers at %0t ps\n", $realtime);	     
           $fdisplay(reg_out_file, "Info: Accessing PHY management registers\n\n");
           $display("Info: Reading from PCS Rx Word Aligner Status Register (Address = 086h)\n");	     
           $fdisplay(reg_out_file, "Info: Reading from PCS Rx Word Aligner Status Register (Address = 086h)\n");	     
			  
			  
			  
		   $display ( "Info: Reconfig Phase has started...\n");
		   if (test_f)
		     begin
			  	if (pio_out[6:0] == 7'b1111111)
			  begin
			     $fdisplay (reg_out_file, "VOD, Pre-emphasis, Postatp1, postatp2, eqdcgain and PreCDR, PostCDR loopback Write/read operation through basic is working");
			     $display ("VOD, Pre-emphasis, Posttap1, posttap2, eqdcgain and PreCDR, PostCDR loopback is working");
			  end
			else
			  begin
			     if (pio_out[0] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! VOD data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end

			     if (pio_out[1] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! Preemphasis pretap (0t)  data read is not correct");
				  $display ("Error! Preemphasis pretap (0t)  data read is not correct");				  

			       end

			     if (pio_out[2] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! Preemphasis posttap2 (2t)  data read is not correct");
				  $display ("Error! Preemphasis posttap2 (2t)  data read is not correct");
			       end

			     if (pio_out[3] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! Preemphasis posttap1 (1t)  data read is not correct");
				  $display ("Error! Preemphasis posttap2 (1t)  data read is not correct");
			       end

			     if (pio_out[4] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! RX EQDCGAIN data read is not correct");
				  $display ("Error! RX EQDCGAIM read is not correct");
			       end

			     if (pio_out[5] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! VOD data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end

			     if (pio_out[6] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! RSLB PRECDR loopback data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end

			     if (pio_out[7] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! RSLB POSTCDR loopback data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end
			  end // else: !if(pio_out[6:0] == 7'b1111111)
		     end // if (test_f)
		  end
   end

	

   //PCS register read status for Custom PHY IP1
   always @ (ready_for_mgmt, rx_syncstatus_read1)
   begin 
     if (ready_for_mgmt)
     begin
       if (rx_syncstatus_read1 == 2'b00)
       begin
           $display("Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b at %0tps\n", rx_syncstatus_read1, $realtime);	     
           $fdisplay(reg_out_file, "Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b\n", rx_syncstatus_read1);
       end
     end
     if (rx_syncstatus_read1 == 2'b11)
     begin
           $display("Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b at %0tps\n", rx_syncstatus_read1, $realtime);	     
           $fdisplay(reg_out_file, "Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b\n", rx_syncstatus_read1);
           $display("Info: Continue sending PRBS data......\n\n");	     
           $fdisplay(reg_out_file, "Info: Continue sending PRBS data......\n\n");
     end
   end
	
	//PCS register read status for Custom PHY IP2
   always @ (ready_for_mgmt, rx_syncstatus_read2)
   begin 
     if (ready_for_mgmt)
     begin
       if (rx_syncstatus_read2 == 2'b00)
       begin
           $display("Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b at %0tps\n", rx_syncstatus_read2, $realtime);	     
           $fdisplay(reg_out_file, "Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b\n", rx_syncstatus_read2);
       end
     end
     if (rx_syncstatus_read2 == 2'b11)
     begin
           $display("Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b at %0tps\n", rx_syncstatus_read2, $realtime);	     
           $fdisplay(reg_out_file, "Info: Rx Word Aligner Status (rx_syncstatus) = phy_mgmt_readdata[5:4] = %b\n", rx_syncstatus_read2);
           $display("Info: Continue sending PRBS data......\n\n");	     
           $fdisplay(reg_out_file, "Info: Continue sending PRBS data......\n\n");
     end
   end
	
/*	always @(ready_for_mgmt)
		begin
		 $display ( "Info: Reconfig Phase has started...\n");
		   if (test_f)
		     begin
			  	if (pio_out[6:0] == 7'b1111111)
			  begin
			     $fdisplay (reg_out_file, "VOD, Pre-emphasis, Postatp1, postatp2, eqdcgain and PreCDR, PostCDR loopback Write/read operation through basic is working");
			     $display ("VOD, Pre-emphasis, Posttap1, posttap2, eqdcgain and PreCDR, PostCDR loopback is working");
			  end
			else
			  begin
			     if (pio_out[0] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! VOD data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end

			     if (pio_out[1] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! Preemphasis pretap (0t)  data read is not correct");
				  $display ("Error! Preemphasis pretap (0t)  data read is not correct");				  

			       end

			     if (pio_out[2] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! Preemphasis posttap2 (2t)  data read is not correct");
				  $display ("Error! Preemphasis posttap2 (2t)  data read is not correct");
			       end

			     if (pio_out[3] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! Preemphasis posttap1 (1t)  data read is not correct");
				  $display ("Error! Preemphasis posttap2 (1t)  data read is not correct");
			       end

			     if (pio_out[4] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! RX EQDCGAIN data read is not correct");
				  $display ("Error! RX EQDCGAIM read is not correct");
			       end

			     if (pio_out[5] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! VOD data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end

			     if (pio_out[6] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! RSLB PRECDR loopback data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end

			     if (pio_out[7] == 1'b0)
			       begin
				  $fdisplay (reg_out_file, "Error! RSLB POSTCDR loopback data read is not correct");
				  $display ("Error! VOD data read is not correct");
			       end
			  end // else: !if(pio_out[6:0] == 7'b1111111)
		     end // if (test_f)
		end // always @ (*)
	
*/
endmodule // top_tb

   
